@@ -121,6 +121,7 @@ static int read_cb_cli(struct osmo_stream_cli *cli)
 		osmo_stream_cli_set_data(cli, msg);
 		CLI_SND(cli, "Doh, responding to server :-D");
 	}
+	msgb_free(msg);
 
 	return 0;
 }
@@ -299,6 +300,7 @@ int read_cb_srv(struct osmo_stream_srv *srv)
 			   than it must be subsequent (after reconnect) call */
 			request_test_stop(srv);
 		}
+		msgb_free(msg);
 		osmo_stream_srv_destroy(srv);
 		return -EINVAL;
 	} else {
@@ -340,10 +342,17 @@ static void test_recon(void *ctx, const char *host, unsigned port, unsigned step
 		       bool autoreconnect)
 {
 	struct timeval tv;
-	struct osmo_stream_cli *cli = make_client(ctx, host, port, autoreconnect);
+	struct osmo_stream_cli *cli;
+	if (osmo_stream_srv_link_open(lnk) < 0) {
+		printf("Unable to open server\n");
+		osmo_stream_srv_link_destroy(lnk);
+		return;
+	}
+	cli = make_client(ctx, host, port, autoreconnect);
 	if (!cli)
 		return;
 
+
 	printf("=======================================\n");
 	printf("Client/Server entering %s event loop...\n", ASTR(autoreconnect));
 	printf("=======================================\n");
@@ -368,6 +377,7 @@ static void test_recon(void *ctx, const char *host, unsigned port, unsigned step
 	}
 
 	osmo_stream_cli_destroy(cli);
+	osmo_stream_srv_link_close(lnk);
 	printf("{%lu.%06lu} %s test complete.\n\n", tv.tv_sec, tv.tv_usec, ASTR(autoreconnect));
 }
 
@@ -384,12 +394,17 @@ static const uint8_t ipac_msg_pong[] = {
 	IPAC_PROTO_IPACCESS,
 	IPAC_MSGT_PONG
 };
+#define IPAC_MSG_IDREQ_PAYLOAD_INITIALIZER \
+	IPAC_MSGT_ID_GET, \
+	0x01, IPAC_IDTAG_UNITNAME
+static const uint8_t ipac_msg_idreq_payload[] = {
+	IPAC_MSG_IDREQ_PAYLOAD_INITIALIZER
+};
 #define IPAC_MSG_ID_REQ_LEN 0x03
 static const uint8_t ipac_msg_idreq[] = {
 	0x00, IPAC_MSG_ID_REQ_LEN,
 	IPAC_PROTO_IPACCESS,
-	IPAC_MSGT_ID_GET,
-	0x01, IPAC_IDTAG_UNITNAME
+	IPAC_MSG_IDREQ_PAYLOAD_INITIALIZER
 };
 #define ipac_msg_idreq_third (sizeof(ipac_msg_idreq)/3)
 #define ipac_msg_idreq_last_third (sizeof(ipac_msg_idreq) - 2 * ipac_msg_idreq_third)
@@ -487,19 +502,23 @@ static void send_last_third(void *osmo_stream_cli_arg)
 
 static struct osmo_timer_list fragmented_send_tl_cli;
 
-static int test_segm_ipa_stream_srv_cli_read_cb(struct osmo_stream_cli *osc, struct msgb *msg)
+static int test_segm_ipa_stream_srv_cli_read_cb(struct osmo_stream_cli *osc, int res, struct msgb *msg)
 {
 	unsigned char *data;
-	struct ipa_head *h = (struct ipa_head *) msg->data;
-	uint8_t ipac_msg_type = ((uint8_t *)h)[sizeof(struct ipa_head)];
-	struct msgb *reply = msgb_alloc_headroom(128, 0, "IPA reply");
-	if (reply == NULL) {
-		fprintf(stderr, "Cannot allocate message\n");
-		return -ENOMEM;
+	struct ipa_head *h = (struct ipa_head *) msg->l1h;
+	uint8_t ipac_msg_type = *msg->data;
+	struct msgb *reply;
+
+	if (res < 0) {
+		fprintf(stderr, "cannot receive message (res = %d)\n", res);
+		msgb_free(msg);
+		return -ENOMSG;
 	}
-	LOGCLI(osc, "Received message from stream (total len = %" PRIu16 ")\n", msgb_length(msg));
+
+	LOGCLI(osc, "Received message from stream (payload len = %" PRIu16 ")\n", msgb_length(msg));
 	if (ipac_msg_type < 0 || 5 < ipac_msg_type) {
 		fprintf(stderr, "Received unexpected IPAC message type %"PRIu8"\n", ipac_msg_type);
+		msgb_free(msg);
 		return -ENOMSG;
 	}
 	LOGCLI(osc, "\tType: %s\n", IPAC_MSG_TYPES[ipac_msg_type]);
@@ -507,6 +526,11 @@ static int test_segm_ipa_stream_srv_cli_read_cb(struct osmo_stream_cli *osc, str
 		LOGCLI(osc, "Got IPAC_MSGT_ID_GET from server\n");
 		LOGCLI(osc, "[(%u + 2/3) -cli] Appending: Second third of IPAC_MSGT_ID_GET\n",
 		       test_segm_ipa_stream_srv_msglognum_cli);
+		reply = msgb_alloc_headroom(128, 0, "IPA reply");
+		if (reply == NULL) {
+			fprintf(stderr, "Cannot allocate message\n");
+			return -ENOMEM;
+		}
 		data = msgb_put(reply, ipac_msg_idreq_third);
 		memcpy(data, ipac_msg_idreq + ipac_msg_idreq_third,
 		       ipac_msg_idreq_third);
@@ -522,17 +546,14 @@ static int test_segm_ipa_stream_srv_cli_read_cb(struct osmo_stream_cli *osc, str
 		LOGCLI(osc, "\texpected=%s\n",
 		       osmo_hexdump(ipac_msg_idresp, sizeof(ipac_msg_idresp)));
 	}
+	msgb_free(msg);
 	printf("\n");
 	return 0;
 }
 
-static void *test_segm_ipa_stream_srv_run_client(void)
+struct osmo_stream_cli *test_segm_ipa_stream_srv_run_client(void *ctx)
 {
-	struct osmo_stream_cli *osc;
-	void *ctx = talloc_named_const(NULL, 0, __func__);
-
-	(void) msgb_talloc_ctx_init(ctx, 0);
-	osc = osmo_stream_cli_create(ctx);
+	struct osmo_stream_cli *osc = osmo_stream_cli_create(ctx);
 	if (osc == NULL) {
 		fprintf(stderr, "osmo_stream_cli_create_iofd()\n");
 		return NULL;
@@ -541,55 +562,67 @@ static void *test_segm_ipa_stream_srv_run_client(void)
 	osmo_stream_cli_set_local_port(osc, 8977);
 	osmo_stream_cli_set_port(osc, 1111);
 	osmo_stream_cli_set_connect_cb(osc, test_segm_ipa_stream_srv_cli_connect_cb);
-	osmo_stream_cli_set_data(osc, ctx);
 	osmo_stream_cli_set_read_cb2(osc, test_segm_ipa_stream_srv_cli_read_cb);
 	osmo_stream_cli_set_nodelay(osc, true);
 	if (osmo_stream_cli_open(osc) < 0) {
 		fprintf(stderr, "Cannot open stream client\n");
 		return NULL;
 	}
+	osmo_stream_cli_set_segmentation_cb(osc, osmo_ipa_segmentation_cb);
 
-	return NULL;
+	return osc;
 }
 
-int test_segm_ipa_stream_srv_srv_read_cb(struct osmo_stream_srv *conn, struct msgb *msg)
+int test_segm_ipa_stream_srv_srv_read_cb(struct osmo_stream_srv *conn, int res, struct msgb *msg)
 {
 	static unsigned msgnum_srv = 0;
 	struct ipa_head *ih = (struct ipa_head *)msg->l1h;
 	unsigned char *data;
 	struct msgb *m;
-	uint8_t *msgt = msg->l2h; /* Octet right after IPA header */
+	uint8_t msgt;
+
+	if (res <= 0) {
+		if (res < 0)
+			LOGSRV(conn, "cannot receive message: %s\n", strerror(-res));
+		else
+			LOGSRV(conn, "client closed connection\n");
+		msgb_free(msg);
+		osmo_stream_srv_destroy(conn);
+		return -EBADF;
+	}
+
 	LOGSRV(conn, "[%u-srv] Received IPA message from stream (payload len = %" PRIu16 ")\n",
 	       ++msgnum_srv, msgb_length(msg));
 	LOGSRV(conn, "\tmsg buff data (including stripped headers): %s\n",
 	       osmo_hexdump((unsigned char *)ih, osmo_ntohs(ih->len) + sizeof(*ih)));
 	LOGSRV(conn, "\tIPA payload: %s\n", osmo_hexdump(ih->data, osmo_ntohs(ih->len)));
-	LOGSRV(conn, "\tType: %s\n", IPAC_MSG_TYPES[*msgt]);
+
+	msgt = *msg->l2h; /* Octet right after IPA header */
+	LOGSRV(conn, "\tType: %s\n", IPAC_MSG_TYPES[msgt]);
 	LOGSRV(conn, "\t(msg dump: %s)\n", osmo_hexdump(msg->l1h, msg->len + sizeof(struct ipa_head)));
-	if (*msgt == IPAC_MSGT_ID_RESP) { /*  */
+
+	msgb_free(msg);
+
+	if (msgt == IPAC_MSGT_ID_RESP) { /*  */
 		LOGSRV(conn, "Send IPAC_MSGT_ID_GET to trigger client to send next third\n\n");
-		m = msgb_alloc_headroom(128, 0, "IPA messages");
+		m = osmo_ipa_msg_alloc(128);
 		if (m == NULL) {
 			fprintf(stderr, "Cannot allocate message\n");
 			return -ENOMEM;
 		}
-		put_ipa_msg(data, m, ipac_msg_idreq);
+		put_ipa_msg(data, m, ipac_msg_idreq_payload);
+		osmo_ipa_msg_push_headers(m, IPAC_PROTO_IPACCESS, -1);
 		osmo_stream_srv_send(conn, m);
-	} else if (msgnum_srv == 7 && *msgt == IPAC_MSGT_PONG) {
+	} else if (msgnum_srv == 7 && msgt == IPAC_MSGT_PONG) {
 		test_segm_ipa_stream_srv_all_msgs_processed = true;
+		osmo_stream_srv_destroy(conn);
 	}
 	return 0;
 }
 
-static int test_segm_ipa_stream_srv_srv_close_cb(struct osmo_stream_srv *conn)
-{
-	osmo_stream_srv_set_segmentation_cb(conn, NULL);
-	return 0;
-}
-
 static int test_segm_ipa_stream_srv_srv_accept_cb(struct osmo_stream_srv_link *srv, int fd)
 {
-	void *ctx = talloc_named_const(NULL, 0, __func__);
+	void *ctx = osmo_stream_srv_link_get_data(srv);
 	struct osmo_stream_srv *oss =
 		osmo_stream_srv_create2(ctx, srv, fd, NULL);
 	if (oss == NULL) {
@@ -598,13 +631,13 @@ static int test_segm_ipa_stream_srv_srv_accept_cb(struct osmo_stream_srv_link *s
 	}
 	osmo_stream_srv_set_segmentation_cb(oss, osmo_ipa_segmentation_cb);
 	osmo_stream_srv_set_read_cb(oss, test_segm_ipa_stream_srv_srv_read_cb);
-	osmo_stream_srv_set_closed_cb(oss, test_segm_ipa_stream_srv_srv_close_cb);
 	return 0;
 }
 
 static void test_segm_ipa_stream_srv_run(void *ctx, const char *host, unsigned port,
 				  struct osmo_stream_srv_link *srv)
 {
+	struct osmo_stream_cli *osc;
 	const char *testname = "test_segm_ipa_stream_srv";
 	osmo_stream_srv_link_set_accept_cb(srv,
 		test_segm_ipa_stream_srv_srv_accept_cb);
@@ -612,7 +645,7 @@ static void test_segm_ipa_stream_srv_run(void *ctx, const char *host, unsigned p
 		printf("Unable to open server\n");
 		exit(1);
 	}
-	test_segm_ipa_stream_srv_run_client();
+	osc = test_segm_ipa_stream_srv_run_client(ctx);
 
 	printf("______________________________________Running test %s______________________________________\n", testname);
 	alarm(2);
@@ -623,6 +656,9 @@ static void test_segm_ipa_stream_srv_run(void *ctx, const char *host, unsigned p
 	}
 	alarm(0);
 	printf("==================================Test %s complete========================================\n\n", testname);
+	if (osc)
+		osmo_stream_cli_destroy(osc);
+	osmo_stream_srv_link_close(srv);
 }
 
 static void sigalarm_handler(int _foo)
@@ -631,6 +667,227 @@ static void sigalarm_handler(int _foo)
 	exit(EXIT_FAILURE);
 }
 
+static struct osmo_timer_list fragmented_send_tl_srv;
+static struct osmo_timer_list fragmented_send_tl_srv_destroy;
+
+static unsigned test_segm_ipa_stream_cli_srv_msglognum = 0;
+
+/* Like CLI_APPEND_MSG, but for server side */
+#define SRV_APPEND_MSG(OSMO_STREAM_SRV_PTR, UCHAR_PTR_DST, STRUCT_MSGB_PTR, SRC_IPAC_MSG_BUF) do {\
+	LOGSRV(OSMO_STREAM_SRV_PTR, "[%u-srv] Appending msg of type %s into buffer\n",\
+		++test_segm_ipa_stream_cli_srv_msglognum, IPAC_MSG_TYPES[SRC_IPAC_MSG_BUF[IPAC_MSGT_OFFSET]]);\
+	LOGSRV(OSMO_STREAM_SRV_PTR, "\t(msg dump: %s)\n", osmo_hexdump(SRC_IPAC_MSG_BUF,\
+	       sizeof(SRC_IPAC_MSG_BUF)));\
+	put_ipa_msg(UCHAR_PTR_DST, STRUCT_MSGB_PTR, SRC_IPAC_MSG_BUF);\
+} while (0)
+
+static void destroy_conn(void *osmo_stream_srv_arg)
+{
+	osmo_stream_srv_destroy(osmo_stream_srv_arg);
+}
+
+static void send_last_third_srv(void *osmo_stream_srv_arg)
+{
+	struct osmo_stream_srv *oss = osmo_stream_srv_arg;
+	unsigned char *data;
+	struct msgb *reply = msgb_alloc_headroom(128, 0, "IPA delayed reply");
+
+	LOGSRV(oss, "Delay for sending last third of message is over\n");
+	if (reply == NULL) {
+		fprintf(stderr, "Cannot allocate message\n");
+		return;
+	}
+	LOGSRV(oss, "[%u-srv] Appending: Last third of IPAC_MSGT_ID_GET\n",
+	       ++test_segm_ipa_stream_cli_srv_msglognum);
+	data = msgb_put(reply, ipac_msg_idreq_last_third);
+	memcpy(data, ipac_msg_idreq + 2 * ipac_msg_idreq_third,
+	       ipac_msg_idreq_last_third);
+	/* Append two entire messages */
+	SRV_APPEND_MSG(oss, data, reply, ipac_msg_pong);
+	SRV_APPEND_MSG(oss, data, reply, ipac_msg_pong);
+	LOGSRV(oss, "\tSending:"
+		    "[ Last third of IPAC_MSGT_ID_GET | IPAC_MSGT_PONG | IPAC_MSGT_PONG ]\n");
+	LOGSRV(oss, "\t(msg dump: %s)\n\n", osmo_hexdump(reply->data, reply->len));
+	osmo_stream_srv_send(oss, reply);
+	osmo_timer_setup(&fragmented_send_tl_srv_destroy, destroy_conn, oss);
+	osmo_timer_add(&fragmented_send_tl_srv_destroy);
+	/* 2 select loop iterations needed, timing only 1 will leave the client side hanging while waiting
+	 * to receive the last messages */
+	osmo_timer_schedule(&fragmented_send_tl_srv_destroy, 0, 2);
+}
+
+int test_segm_ipa_stream_cli_srv_read_cb(struct osmo_stream_srv *conn, int res, struct msgb *msg)
+{
+	unsigned char *data;
+	struct ipa_head *h = (struct ipa_head *) msg->l1h;
+	uint8_t ipa_msg_type;
+	struct msgb *reply;
+
+	if (res <= 0) {
+		if (res < 0)
+			LOGSRV(conn, "cannot receive message: %s\n", strerror(-res));
+		else
+			LOGSRV(conn, "client closed connection\n");
+		msgb_free(msg);
+		osmo_stream_srv_destroy(conn);
+		return -EBADF;
+	}
+
+	ipa_msg_type = ((uint8_t *)h)[sizeof(struct ipa_head)];
+
+	reply = msgb_alloc_headroom(128, 0, "IPA reply");
+	if (reply == NULL) {
+		fprintf(stderr, "Cannot allocate message\n");
+		return -ENOMEM;
+	}
+	LOGSRV(conn, "Received message from stream (total len including stripped headers = %lu)\n",
+	       osmo_ntohs(h->len) + sizeof(*h));
+	if (5 < ipa_msg_type) {
+		fprintf(stderr, "Received unexpected IPAC message type %"PRIu8"\n", ipa_msg_type);
+		return -ENOMSG;
+	}
+	LOGSRV(conn, "\tType: %s\n", IPAC_MSG_TYPES[ipa_msg_type]);
+	if (ipa_msg_type == IPAC_MSGT_ID_GET) {
+		LOGSRV(conn, "Got IPAC_MSGT_ID_GET from client\n");
+		LOGSRV(conn, "[(%u + 2/3) -srv] Appending: Second third of IPAC_MSGT_ID_GET\n",
+		       test_segm_ipa_stream_cli_srv_msglognum);
+		data = msgb_put(reply, ipac_msg_idreq_third);
+		memcpy(data, ipac_msg_idreq + ipac_msg_idreq_third,
+		       ipac_msg_idreq_third);
+		LOGSRV(conn, "\tSending: Second third of IPAC_MSGT_ID_GET\n");
+		LOGSRV(conn, "\t(msg dump: %s)\n", osmo_hexdump(reply->data, reply->len));
+		osmo_stream_srv_send(conn, reply);
+		osmo_timer_setup(&fragmented_send_tl_srv, send_last_third_srv, conn);
+		osmo_timer_add(&fragmented_send_tl_srv);
+		osmo_timer_schedule(&fragmented_send_tl_srv, 0, 125000);
+	} else if (ipa_msg_type == IPAC_MSGT_ID_RESP) {
+		LOGSRV(conn, "\tresult=  %s\n",
+		       osmo_hexdump((const unsigned char *)h, sizeof(*h) + h->len));
+		LOGSRV(conn, "\texpected=%s\n",
+		       osmo_hexdump(ipac_msg_idresp, sizeof(ipac_msg_idresp)));
+	}
+	printf("\n");
+	return 0;
+}
+
+static int test_segm_ipa_stream_cli_srv_accept_cb(struct osmo_stream_srv_link *srv, int fd)
+{
+	void *ctx = osmo_stream_srv_link_get_data(srv);
+	struct osmo_stream_srv *oss =
+		osmo_stream_srv_create2(ctx, srv, fd, NULL);
+	unsigned char *data;
+	struct msgb *m = msgb_alloc_headroom(128, 0, "IPA messages");
+	if (oss == NULL) {
+		fprintf(stderr, "Error while creating connection\n");
+		return -1;
+	}
+	if (m == NULL) {
+		fprintf(stderr, "Cannot allocate message\n");
+		return -ENOMEM;
+	}
+	osmo_stream_srv_set_segmentation_cb(oss, osmo_ipa_segmentation_cb);
+	osmo_stream_srv_set_read_cb(oss, test_segm_ipa_stream_cli_srv_read_cb);
+
+	/* Send 4 and 1/3 messages, as done analogously in test_segm_ipa_stream_srv_cli_connect_cb() */
+	/* Append 4 */
+	SRV_APPEND_MSG(oss, data, m, ipac_msg_ping);
+	SRV_APPEND_MSG(oss, data, m, ipac_msg_pong);
+	SRV_APPEND_MSG(oss, data, m, ipac_msg_ping);
+	SRV_APPEND_MSG(oss, data, m, ipac_msg_idresp);
+	/* Append 1/3 */
+	LOGSRV(oss, "[(0%u + 1/3)-srv] Appending 1st third of msg of type %s into buffer\n",
+	       test_segm_ipa_stream_cli_srv_msglognum, IPAC_MSG_TYPES[ipac_msg_idreq[3]]);
+	LOGSRV(oss, "\t(dump: %s)\n", osmo_hexdump(ipac_msg_idreq, ipac_msg_idreq_third));
+	data = msgb_put(m, ipac_msg_idreq_third);
+	memcpy(data, ipac_msg_idreq, ipac_msg_idreq_third);
+
+	LOGSRV(oss, "Sending 4 + 1/3 messages as one:\n");
+	LOGSRV(oss, "\t(msg dump: %s)\n\n", osmo_hexdump(m->data, m->len));
+	osmo_stream_srv_send(oss, m);
+	return 0;
+}
+
+static bool test_segm_ipa_stream_cli_all_msgs_processed = false;
+
+static int test_segm_ipa_stream_cli_cli_read_cb(struct osmo_stream_cli *osc, int res, struct msgb *msg)
+{
+	static unsigned msgnum_cli = 0;
+	unsigned char *data;
+	struct msgb *m;
+	uint8_t *msgt = msg->data;
+	LOGCLI(osc, "[%u-cli] Received message from stream (len = %" PRIu16 ")\n",
+	       ++msgnum_cli, msgb_length(msg));
+	LOGCLI(osc, "\tmsg buff data: %s\n", osmo_hexdump(msg->data, msg->len));
+	LOGCLI(osc, "\tIPA payload: %s\n", osmo_hexdump(msg->data, msg->len));
+	LOGCLI(osc, "\tType: %s\n", IPAC_MSG_TYPES[*msgt]);
+	LOGCLI(osc, "\t(msg dump (including stripped headers): %s)\n",
+	       osmo_hexdump(msg->l1h, sizeof(struct ipa_head) + msg->len));
+	if (*msgt == IPAC_MSGT_ID_RESP) {
+		LOGCLI(osc, "Send IPAC_MSGT_ID_GET to trigger server to send next third\n\n");
+		m = msgb_alloc_headroom(128, sizeof(struct ipa_head) +
+					     sizeof(struct ipa_head_ext), "IPA messages");
+		if (m == NULL) {
+			fprintf(stderr, "Cannot allocate message\n");
+			return -ENOMEM;
+		}
+		put_ipa_msg(data, m, ipac_msg_idreq_payload);
+		osmo_ipa_msg_push_headers(m, IPAC_PROTO_IPACCESS, -1);
+		osmo_stream_cli_send(osc, m);
+	} else if (msgnum_cli == 7 && *msgt == IPAC_MSGT_PONG) {
+		test_segm_ipa_stream_cli_all_msgs_processed = true;
+	}
+	return 0;
+}
+
+static struct osmo_stream_cli *test_segm_ipa_stream_cli_run_client(void *ctx)
+{
+	struct osmo_stream_cli *osc = osmo_stream_cli_create(ctx);
+	if (osc == NULL) {
+		fprintf(stderr, "osmo_stream_cli_create_iofd()\n");
+		return NULL;
+	}
+	osmo_stream_cli_set_addr(osc, "127.0.0.11");
+	osmo_stream_cli_set_local_port(osc, 8977);
+	osmo_stream_cli_set_port(osc, 1112);
+	osmo_stream_cli_set_read_cb2(osc, test_segm_ipa_stream_cli_cli_read_cb);
+	osmo_stream_cli_set_nodelay(osc, true);
+	osmo_stream_cli_set_segmentation_cb(osc, osmo_ipa_segmentation_cb);
+	if (osmo_stream_cli_open(osc) < 0) {
+		fprintf(stderr, "Cannot open stream client\n");
+		return NULL;
+	}
+
+	return osc;
+}
+
+static void test_segm_ipa_stream_cli_run(void *ctx, const char *host, unsigned port,
+				  struct osmo_stream_srv_link *srv)
+{
+	const char *testname = "test_segm_ipa_stream_cli";
+	struct osmo_stream_cli *osc = NULL;
+	osmo_stream_srv_link_set_accept_cb(srv,
+		test_segm_ipa_stream_cli_srv_accept_cb);
+	osmo_stream_srv_link_set_port(srv, 1112);
+	if (osmo_stream_srv_link_open(srv) < 0) {
+		printf("Unable to open server\n");
+		exit(1);
+	}
+	osc = test_segm_ipa_stream_cli_run_client(ctx);
+
+	printf("______________________________________Running test %s______________________________________\n", testname);
+	alarm(2);
+
+	while (!test_segm_ipa_stream_cli_all_msgs_processed) {
+		osmo_gettimeofday_override_add(0, 1); /* small increment to easily spot iterations */
+		osmo_select_main(1);
+	}
+	alarm(0);
+	printf("==================================Test %s complete========================================\n\n", testname);
+	if (osc)
+		osmo_stream_cli_destroy(osc);
+	osmo_stream_srv_link_close(srv);
+}
+
 int main(void)
 {
 
@@ -648,7 +905,6 @@ int main(void)
 	osmo_gettimeofday_override_time.tv_sec = 2;
 	osmo_gettimeofday_override_time.tv_usec = 0;
 
-	msgb_talloc_ctx_init(tall_test, 0);
 	osmo_init_logging2(tall_test, &osmo_stream_test_log_info);
 	log_set_log_level(osmo_stderr_target, LOGL_INFO);
 	log_set_use_color(osmo_stderr_target, 0);
@@ -667,20 +923,19 @@ int main(void)
 	osmo_stream_srv_link_set_addr(srv, host);
 	osmo_stream_srv_link_set_port(srv, port);
 	osmo_stream_srv_link_set_accept_cb(srv, accept_cb_srv);
-
-	if (osmo_stream_srv_link_open(srv) < 0) {
-		printf("Unable to open server\n");
-		osmo_stream_srv_link_destroy(srv);
-		return EXIT_FAILURE;
-	}
+	osmo_stream_srv_link_set_nodelay(srv, true);
 
 	test_recon(tall_test, host, port, 12, srv, true);
 	test_recon(tall_test, host, port, 8, srv, false);
 
+	osmo_stream_srv_link_set_data(srv, tall_test);
 	test_segm_ipa_stream_srv_run(tall_test, host, port, srv);
+	test_segm_ipa_stream_cli_run(tall_test, host, port, srv);
 
 	printf("Stream tests completed\n");
 
 	osmo_stream_srv_link_destroy(srv);
+	log_fini();
+	talloc_free(tall_test);
 	return EXIT_SUCCESS;
 }
